@@ -2,22 +2,24 @@ import io
 import mimetypes
 import os
 import posixpath
+import tempfile
 import threading
 from datetime import datetime, timedelta
 from gzip import GzipFile
 from tempfile import SpooledTemporaryFile
 from urllib.parse import parse_qsl, urlsplit
 
+from django.contrib.staticfiles.storage import ManifestFilesMixin
 from django.core.exceptions import ImproperlyConfigured, SuspiciousOperation
 from django.core.files.base import File
 from django.utils.deconstruct import deconstructible
-from django.utils.encoding import filepath_to_uri, force_bytes
+from django.utils.encoding import filepath_to_uri
 from django.utils.timezone import is_naive, make_naive
 
 from storages.base import BaseStorage
 from storages.utils import (
     check_location, get_available_overwrite_name, lookup_env, safe_join,
-    setting,
+    setting, to_bytes,
 )
 
 try:
@@ -75,7 +77,6 @@ else:
 
 @deconstructible
 class S3Boto3StorageFile(File):
-
     """
     The default file object used by the S3Boto3Storage backend.
 
@@ -92,7 +93,6 @@ class S3Boto3StorageFile(File):
     order to properly write the file to S3. Be sure to close the file
     in your application.
     """
-    buffer_size = setting('AWS_S3_FILE_BUFFER_SIZE', 5242880)
 
     def __init__(self, name, mode, storage, buffer_size=None):
         if 'r' in mode and 'w' in mode:
@@ -113,8 +113,7 @@ class S3Boto3StorageFile(File):
         # Amazon allows up to 10,000 parts.  The default supports uploads
         # up to roughly 50 GB.  Increase the part size to accommodate
         # for files larger than this.
-        if buffer_size is not None:
-            self.buffer_size = buffer_size
+        self.buffer_size = buffer_size or setting('AWS_S3_FILE_BUFFER_SIZE', 5242880)
         self._write_counter = 0
 
     @property
@@ -161,7 +160,7 @@ class S3Boto3StorageFile(File):
             )
         if self.buffer_size <= self._buffer_file_size:
             self._flush_write_buffer()
-        bstr = force_bytes(content)
+        bstr = to_bytes(content)
         self._raw_bytes_written += len(bstr)
         return super().write(bstr)
 
@@ -414,11 +413,8 @@ class S3Boto3Storage(BaseStorage):
         #  This means each time a file is compressed it changes even if the other contents don't change
         #  For S3 this defeats detection of changes using MD5 sums on gzipped files
         #  Fixing the mtime at 0.0 at compression time avoids this problem
-        zfile = GzipFile(mode='wb', fileobj=zbuf, mtime=0.0)
-        try:
-            zfile.write(force_bytes(content.read()))
-        finally:
-            zfile.close()
+        with GzipFile(mode='wb', fileobj=zbuf, mtime=0.0) as zfile:
+            zfile.write(to_bytes(content.read()))
         zbuf.seek(0)
         # Boto 2 returned the InMemoryUploadedFile with the file pointer replaced,
         # but Boto 3 seems to have issues with that. No need for fp.name in Boto3
@@ -587,3 +583,22 @@ class S3Boto3Storage(BaseStorage):
         if self.file_overwrite:
             return get_available_overwrite_name(name, max_length)
         return super().get_available_name(name, max_length)
+
+
+class S3StaticStorage(S3Boto3Storage):
+    """Querystring auth must be disabled so that url() returns a consistent output."""
+    querystring_auth = False
+
+
+class S3ManifestStaticStorage(ManifestFilesMixin, S3StaticStorage):
+    """Copy the file before saving for compatibility with ManifestFilesMixin
+    which does not play nicely with boto3 automatically closing the file.
+
+    See: https://github.com/boto/s3transfer/issues/80#issuecomment-562356142
+    """
+
+    def _save(self, name, content):
+        content.seek(0)
+        with tempfile.SpooledTemporaryFile() as tmp:
+            tmp.write(content.read())
+            return super()._save(name, tmp)
